//
//  TJCutImageViewController.m
//  smallTest
//
//  Created by CZHH-12 on 2017/8/1.
//  Copyright © 2017年 CZHH-12. All rights reserved.
//
#import "TJCutImageViewController.h"
#import "TJCutRangeView.h"
typedef void(^handler)(UIImage *image);

@interface TJCutImageViewController ()<UIScrollViewDelegate, UIGestureRecognizerDelegate>
{
    CGFloat _captureWith;//裁切范围
    
    CGFloat _browser_width;//屏幕宽
    CGFloat _browser_height;//屏幕高
    
    CGFloat _horSpace;//水平扩展范围
    CGFloat _verSpace;//竖直扩展范围
    
    CGFloat _fitScale;//自适应缩放率
}
@property (nonatomic, strong) UIScrollView *scaleView;

@property (nonatomic, strong) UIImageView *pics;

@property (nonatomic, strong) UIImage *sourceImg;//原图片


@property (nonatomic, strong) UIButton *sureBtn;
@property (nonatomic, strong) UIButton *cancelBtn;

//回掉包
@property (nonatomic, copy) handler block;

//剪切框
@property (nonatomic, strong) TJCutRangeView *rangeView;

@end

@implementation TJCutImageViewController
#pragma mark - 初始化数据
- (void)initData {
    
    _captureWith = [UIScreen mainScreen].bounds.size.width-40;
    
    _browser_width = self.view.frame.size.width;
    _browser_height = self.view.frame.size.height;
    
    _horSpace = (_browser_width - _captureWith);
    _verSpace = (_browser_height - _captureWith);
    
    [self changeFrameWithImage:self.sourceImg];
    
    if (_pics.frame.size.width >_pics.frame.size.height) {
        _fitScale = _captureWith/_pics.frame.size.height;
    }else {
        _fitScale = _captureWith/_pics.frame.size.width;
    }
    
    //设置最小缩放率
    _scaleView.minimumZoomScale = _fitScale;
    [_scaleView setZoomScale:_fitScale];
    
}

-(void)changeFrameWithImage:(UIImage *)image
{
    
    CGFloat height = image.size.height / image.size.width * _browser_width;
    self.pics.frame = CGRectMake(0, 0, _browser_width, height);
    
    [self changeContent];
    
    CGFloat width = _scaleView.contentSize.width;
    CGFloat heights = _scaleView.contentSize.height;
    
    self.pics.center = CGPointMake(width / 2, heights / 2);
    
    CGPoint offset = CGPointMake((width - _browser_width)/2, (heights - _browser_height)/2);
    
    _scaleView.contentOffset = offset;
    self.pics.image = image;
    
}

- (void)changeContent {
    
    CGSize size = CGSizeMake(_pics.frame.size.width + _horSpace, _pics.frame.size.height + _verSpace);
    self.scaleView.contentSize = size;
    
}

#pragma mark - 界面加载完成
- (void)viewDidLoad {
    [super viewDidLoad];
    self.automaticallyAdjustsScrollViewInsets = NO;
    
    [self initData];
    
    UIButton *sureBtn = [[UIButton alloc]initWithFrame:CGRectMake(20, self.view.frame.size.height - 50, 50, 50)];
    [sureBtn setTitle:@"确定" forState:UIControlStateNormal];
    [sureBtn addTarget:self action:@selector(sure) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:sureBtn];
    
    UIButton *cancelBtn = [[UIButton alloc]initWithFrame:CGRectMake(self.view.frame.size.width - 50 -20, self.view.frame.size.height - 50, 50, 50)];
    [cancelBtn addTarget:self action:@selector(cancel) forControlEvents:UIControlEventTouchUpInside];
    [cancelBtn setTitle:@"取消" forState:UIControlStateNormal];
    [self.view addSubview:cancelBtn];
    
    TJCutRangeView *range = [[TJCutRangeView alloc]init];
    range.frame = CGRectMake(0, 0, _captureWith, _captureWith);
    range.center = self.view.center;
    range.backgroundColor = [UIColor clearColor];
    self.rangeView = range;
    [self.view addSubview:range];
    [self.view bringSubviewToFront:range];
}

-(void)setNeedsDisplay
{
    [_scaleView setZoomScale:1.0 animated:NO];
}

-(UIView *)viewForZoomingInScrollView:(UIScrollView *)scrollView
{
    return _pics;
}

-(void)scrollViewDidZoom:(UIScrollView *)scrollView
{
    [self changeContent];
    
    CGFloat xcenter = scrollView.center.x , ycenter = scrollView.center.y;
    //目前contentsize的width是否大于原scrollview的contentsize，如果大于，设置imageview中心x点为contentsize的一半，以固定imageview在该contentsize中心。如果不大于说明图像的宽还没有超出屏幕范围，可继续让中心x点为屏幕中点，此种情况确保图像在屏幕中心。
    xcenter = scrollView.contentSize.width > scrollView.frame.size.width ? (scrollView.contentSize.width)/2 : (xcenter);
    ycenter = scrollView.contentSize.height > scrollView.frame.size.height ? (scrollView.contentSize.height)/2 : (ycenter);
    
    [_pics setCenter:CGPointMake(xcenter, ycenter)];
    
}

- (void)handleSingleFingerEvent:(UITapGestureRecognizer *)sender
{
    
    if (sender.numberOfTapsRequired == 2){
        if (_scaleView.zoomScale > _fitScale) {
            [_scaleView setZoomScale:_fitScale animated:YES];
        } else {
            CGFloat maxScale = _scaleView.maximumZoomScale;
            [_scaleView setZoomScale:maxScale animated:YES];
        }
    }
    
}

#pragma mark - 确定和取消
- (void)cancel {
    
    if (self.presentingViewController) {
        [self dismissViewControllerAnimated:YES completion:nil];
    } else {
        [self.navigationController popViewControllerAnimated:YES];
    }
    
}

- (void)sure {
    
    if (_block) {
        self.rangeView.hidden = YES;
        CGRect rect = CGRectMake(self.rangeView.frame.origin.x, self.rangeView.frame.origin.y, self.rangeView.frame.size.width, self.rangeView.frame.size.height);
        UIImage *image = [self screenImageWithRect:rect];
        NSData *imageData = UIImageJPEGRepresentation(image, 0.5);
        UIImage *resultImg = [UIImage imageWithData:imageData];
        _block(resultImg);
    }
    [self cancel];
    
}

#pragma mark - 懒加载
- (UIScrollView *)scaleView {
    
    if (!_scaleView) {
        _scaleView = [[UIScrollView alloc]initWithFrame:CGRectMake(0, 0, _browser_width, _browser_height)];
        _scaleView.delegate = self;
        _scaleView.maximumZoomScale = 4.0;
        _scaleView.minimumZoomScale = 1;
        _scaleView.bouncesZoom = YES;
        _scaleView.multipleTouchEnabled = YES;
        _scaleView.delegate = self;
        _scaleView.scrollsToTop = NO;
        _scaleView.showsHorizontalScrollIndicator = NO;
        _scaleView.showsVerticalScrollIndicator = NO;
        _scaleView.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
        _scaleView.delaysContentTouches = NO;
        _scaleView.canCancelContentTouches = YES;
        _scaleView.alwaysBounceVertical = NO;
        _scaleView.userInteractionEnabled = YES;
        [self.view addSubview:_scaleView];
    }
    return _scaleView;
    
}

- (UIImageView *)pics {
    
    if (!_pics) {
        _pics = [[UIImageView alloc]init];
        _pics.contentMode = UIViewContentModeScaleAspectFit;
        _pics.userInteractionEnabled = YES;
        [self.scaleView addSubview:_pics];
        
        UITapGestureRecognizer *singleFingerOne = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(handleSingleFingerEvent:)];
        singleFingerOne.delegate = self;
        singleFingerOne.numberOfTapsRequired = 1;
        [_pics addGestureRecognizer:singleFingerOne];
        
        UITapGestureRecognizer *singleFingerTwo = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(handleSingleFingerEvent:)];
        singleFingerTwo.delegate = self;
        singleFingerTwo.numberOfTapsRequired = 2;
        [_pics addGestureRecognizer:singleFingerTwo];
    }
    return _pics;
    
}

#pragma mark - 截取图片
-(UIImage *)screenImageWithRect:(CGRect)imgRect{
    //    CGFloat deviceScale = [UIScreen mainScreen].scale;
    //    直接截取屏幕 模糊不推荐
    //    CGRect cropFrame = imgRect;
    //    CGFloat orgX = cropFrame.origin.x * deviceScale;
    //    CGFloat orgY = cropFrame.origin.y * deviceScale;
    //    CGFloat width = cropFrame.size.width * deviceScale;
    //    CGFloat height = cropFrame.size.height * deviceScale;
    //
    //    CGRect cropRect = CGRectMake(orgX, orgY, width, height);
    //
    //    UIGraphicsBeginImageContextWithOptions([UIApplication sharedApplication].keyWindow.frame.size, NO, deviceScale);
    //    CGContextRef context = UIGraphicsGetCurrentContext();
    //    CGContextSaveGState(context);
    ////    UIRectClip(cropRect);
    //    [[UIApplication sharedApplication].keyWindow.layer renderInContext:context];
    //    UIImage *viewImage = UIGraphicsGetImageFromCurrentImageContext();
    //    UIGraphicsEndImageContext();
    //
    //    CGRect newRect = [_pics convertRect:cropRect fromView:self.view];
    //    UIImage * imgeee = [UIImage imageWithCGImage:CGImageCreateWithImageInRect([_pics.image CGImage], newRect)];
    //
    //    return imgeee;
    
    
    /**************  在原图上截取 计算坐标比较麻烦   ***********/
    
    //获得截取框相对于imageView的坐标大小
    CGRect imageViewRect = [_pics convertRect:imgRect fromView:self.view];
    
    //获得结果不准确 需要进行缩放调整
    imageViewRect.origin = CGPointMake(imageViewRect.origin.x * _scaleView.zoomScale, imageViewRect.origin.y * _scaleView.zoomScale);
    imageViewRect.size = CGSizeMake(_captureWith, _captureWith);
    
    //获得imageView对于原图的缩放率
    CGFloat scale = self.sourceImg.size.width/_pics.frame.size.width;
    
    //获得截取框相对原图上的坐标大小
    CGRect newRect = CGRectMake(imageViewRect.origin.x *scale, imageViewRect.origin.y*scale, imageViewRect.size.width*scale, imageViewRect.size.height*scale);
    
    //创建CGImageRef并从一个上边坐标大小中截取源UIImage的一部分
    CGImageRef cgimg = CGImageCreateWithImageInRect(self.sourceImg.CGImage,newRect);
    
    UIImage *newImage = [UIImage imageWithCGImage:cgimg];
    //注意释放CGImageRef，因为创建方法包含Create
    CGImageRelease(cgimg);
    
    return newImage;
    
}

#pragma mark - 返回图片
- (void)cutImageWithImage:(UIImage *)image
                  Handler:(void(^)(UIImage *image))handler{
    
    self.sourceImg = image;
    self.block = handler;
    
}

@end
